/** \file
 * SmartKey Driver Installation
 */

#ifndef __SKEYINST_H
#define __SKEYINST_H

#ifdef __cplusplus
extern "C" {
#endif

/* Only for internal use */
#ifdef SKEYINST_EXPORTS
#define SKEYINST_LINK
#else
#define SKEYINST_LINK __declspec(dllimport)
#endif
#define SKEYINST_CALL __stdcall

/**
 * Installation of the USB driver.
 * All the required drivers for the USB SmartKey are installed.
 * On Windows NT the USB Phoenix stack is also installed.
 * \note
 * The files skeyusb.inf, skeyusbnt.inf, skeyusb.sys, skeyusbnt.sys and usbdrv.sys must reside in the same directory of the DLL.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyInstallUSB(void);

/**
 * Installation of the Parallel driver.
 * All the required drivers for the Parallel SmartKey are installed.
 * If in the current operating systen no driver is needed, nothing is done and the function return with success.
 * \note
 * The file keyp.sys must reside in the same directory of the DLL.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyInstallPar(void);

/**
 * Installation of the GSS legacy (old gssiofd.sys) driver.
 * \note
 * The file gssiofd.sys must reside in the same directory of the DLL.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyInstallGSSIOFD(void);

/**
 * Installation of the GSS driver.
 * \note
 * The file gssdrv10.sys must reside in the same directory of the DLL.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyInstallGSSUni(void);

/**
 * Uninstallation of the USB driver.
 * The previous installed USB drivers are uninstalled. 
 * If no driver is installed nothing is done and the function returns with success.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyUnInstallUSB(void);

/**
 * Uninstallation of the Parallel driver.
 * The previous installed Parallel drivers are uninstalled.
 * If no driver is installed nothing is done and the function returns with success.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyUnInstallPar(void);

/**
 * Uninstallation of the GSS Legacy (old gssiofd.sys) driver.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyUnInstallGSSIOFD(void);

/**
 * Uninstallation of the GSS driver.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyUnInstallGSSUni(void);

/**
 * Forced uninstallation of the USB driver.
 * The driver is uninstalled with force removing any reference on the filesystem and registry.
 * \note Use only in extreme cases to recover from wrong installation.
 * \note After a forced uninstallation you MUST always reboot.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyForceUnInstallUSB(void);

/**
 * Forced uninstallation of the Parallel driver.
 * The driver is uninstalled with force removing any reference on the filesystem and registry.
 * \note Use only in extreme cases to recover from wrong installation.
 * \note After a forced uninstallation you MUST always reboot.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyForceUnInstallPar(void);

/**
 * Forced uninstallation of the GSS Legacy (old gssiofd.sys) driver.
 * The driver is uninstalled with force removing any reference on the filesystem and registry.
 * \note Use only in extreme cases to recover from wrong installation.
 * \note After a forced uninstallation you MUST always reboot.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyForceUnInstallGSSIOFD(void);

/**
 * Forced uninstallation of the GSS driver.
 * The driver is uninstalled with force removing any reference on the filesystem and registry. 
 * \note Use only in extreme cases to recover from wrong installation.
 * \note After a forced uninstallation you MUST always reboot.
 * \result A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyForceUnInstallGSSUni(void);

/**
 * Get the last Windows error code.
 * \return the last Windows error code like the GetLastError() function.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyGetLastError(void);

/** \name Probing
 * Probing codes used with the SkeyProbe function.
 */
/*@{*/
/** 
 * Check if the USB driver is installed.
 * This command doesn't check if the correct driver is installed, but only if 
 * a generic driver connected at the token is installed. 
 * If the user install manually the "Windows Null Driver", this function reports that a driver is installed.
 */
#define SKEYINST_PROBE_USB_INSTALLED 1

/** 
 * Check if the USB driver is running. 
 * This command doesn't check if the correct driver is running, but only if 
 * a generic driver connected at the token is running. 
 */
#define SKEYINST_PROBE_USB_RUNNING 2

/** 
 * Check if the Parallel driver is installed. 
 */
#define SKEYINST_PROBE_PARALLEL_INSTALLED 3

/** 
 * Check if the Parallel driver is running. 
 */
#define SKEYINST_PROBE_PARALLEL_RUNNING 4

/** 
 * Check if the GSS IOFD driver is installed. 
 */
#define SKEYINST_PROBE_GSSIOFD_INSTALLED 5

/** 
 * Check if the GSS IOFD driver is running.
 */
#define SKEYINST_PROBE_GSSIOFD_RUNNING 6

/** 
 * Check if the GSS UNI driver is installed.
 */
#define SKEYINST_PROBE_GSSUNI_INSTALLED 7

/** 
 * Check if the GSS UNI driver is running.
 */
#define SKEYINST_PROBE_GSSUNI_RUNNING 8

/**
 * Check if the Windows Installer is busy.
 * Until the Windows Installer is busy you cannot call any other function.
 */
#define SKEYINST_PROBE_INSTALL_BUSY 9

/** 
 * Check if the current user has the Adminitrator permission.
 */
#define SKEYINST_PROBE_ADMINISTRATOR_PERMISSION 10

/*@}*/

/**
 * Probe for a driver status information.
 * \param Code A SKEYINST_PROBE_* code.
 * \param Found Output variable. This variable is set only if SKEYINST_OK is returned.
 * \return A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyProbe(DWORD Code, BOOL* Found);

/**
 * Select the log file to use.
 * If you specify a log file, when you call one of the install or uninstall functions
 * a very detailed report of all the operations is generated. 
 * This report can be used to debug problematic installation cases.
 * \param Path Complete path of the log file. Use NULL to disable it.
 * \return A SKEYINST_* code.
 */
SKEYINST_LINK DWORD SKEYINST_CALL SkeyLogFile(const char* Path);

/** \name Error
 * Error codes returned by the all the functions.
 */
/*@{*/

/** 
 * Another installation is in progress. 
 * You must ask at the user to complete any installation process in progress and then retry.
 *
 * A good behaviours is:
 *  - Try the first time
 *  - Wait 5 seconds and retry
 *  - Ask the user to complete and to close any installation window and retry
 *  - Repeat the last step
 *
 * Note that the Windows XP tooltip, shown on the lower right corner of the screen 
 * after an hardware installation, must be closed because is part of the installation 
 * process.
 */
#define SKEYINST_ERROR_WAIT -3 

/** 
 * This operation is unsupported in this operating system.
 * You can't call the function in this operation system.
 */
#define SKEYINST_ERROR_UNSUPPORTED -2 

/**
 * Generic failure.
 * The operation is failed for a generic error. You can call the SkeyGetLastError() function 
 * to get the Windows error code. 
 * If an install operation is done without the Administrator rights the SkeyGetLastError()
 * function returns the valure 5 (ERROR_ACCESS_DENIED).
 */
#define SKEYINST_ERROR_FAIL -1 

/**
 * Operation completed successfully.
 */
#define SKEYINST_OK 0

/**
 * Operation completed successfully, but you must reboot to complete the operation.
 * This code is generally returned after the forced uninstallation.
 */
#define SKEYINST_WARNING_MUST_REBOOT 1

/**
 * Operation completed successfully, but you must insert the token to complete the operation.
 * This code is generally returned after the installation of the USB Driver on Windows 9x/Me/2000/XP.
 */
#define SKEYINST_WARNING_MUST_INSERT 2

/*@}*/

#ifdef __cplusplus
}
#endif

#endif